package io.provis.nexus;

import io.provis.aether.ArtifactResolver;
import io.provis.assembly.action.artifact.UnpackAction;
import io.tesla.proviso.archive.Archiver;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;
import javax.inject.Named;

import org.codehaus.plexus.util.FileUtils;
import org.codehaus.plexus.util.IOUtil;
import org.slf4j.Logger;

import com.google.common.io.ByteStreams;

import de.pdark.decentxml.Document;
import de.pdark.decentxml.Element;
import de.pdark.decentxml.Node;
import de.pdark.decentxml.XMLParser;
import de.pdark.decentxml.XMLWriter;

@Named(NexusProvisioner.ID)
public class NexusProvisioner {

  static final String ID = "nexus";

  @Inject
  private Logger log;

  @Inject
  private ArtifactResolver artifactResolver;

  @Inject
  private Archiver archiver;

  public File provision(NexusProvisioningContext context) throws IOException {

    String version = context.getVersion();
    File installationDirectory = context.getInstallationDirectory();
    File workDirectory = context.getWorkDirectory();

    if (version.length() <= 0) {
      throw new IllegalArgumentException("Nexus version not specified");
    }

    //
    // Grab the Nexus distribution and modify it to run as a WAR in Jetty
    //
    File nexusDistribution = artifactResolver.resolve("org.sonatype.nexus:nexus-oss-webapp:tar.gz:bundle:" + version);
    //
    // Create the installation and work directories
    //
    FileUtils.mkdir(installationDirectory.getAbsolutePath());
    FileUtils.mkdir(workDirectory.getAbsolutePath());

    //
    // Unpack Nexus into the installation directory
    //
    archiver.unarchive(nexusDistribution, installationDirectory, null, null, false, false);

    //
    // Provision any Nexus plugin required
    //
    for (String plugin : context.getPlugins()) {
      addPlugin(plugin, workDirectory);
    }

    InputStream is = getClass().getClassLoader().getResourceAsStream("nexus/conf/security-configuration.xml");
    File securityConfigurationXml = new File(workDirectory, "conf/security-configuration.xml");
    if (is != null) {
      if (!securityConfigurationXml.getParentFile().exists()) {
        securityConfigurationXml.getParentFile().mkdirs();
      }
      OutputStream os = new FileOutputStream(securityConfigurationXml);
      ByteStreams.copy(is, os);
    }

    addRealms(securityConfigurationXml, context.getRealms());
    
    return installationDirectory;
  }

  //
  //  <?xml version="1.0"?>                                                                                                                                       
  //  <security-configuration>                                                                                                                                    
  //    <version>2.0.3</version>                                                                                                                                  
  //    <enabled>true</enabled>                                                                                                                                   
  //    <anonymousAccessEnabled>true</anonymousAccessEnabled>                                                                                                     
  //    <anonymousUsername>anonymous</anonymousUsername>                                                                                                          
  //    <anonymousPassword>{6xW8vwUKYaMHZa5LDPwrNa9+sqlDtm90JwhaeApXN10=}</anonymousPassword>                                                                     
  //    <realms>                                                                                                                                                  
  //      <realm>XmlAuthenticatingRealm</realm>                                                                                                                   
  //      <realm>XmlAuthorizingRealm</realm>                                                                                                                      
  //      <realm>GithubRealm</realm>                                                                                                                              
  //    </realms>                                                                                                                                                 
  //    <securityManager>default</securityManager>                                                                                                                
  //  </security-configuration>
  //
  public void addRealms(File securityConfigurationXml, List<String> realms) throws IOException {
    addRealms(securityConfigurationXml, realms.toArray(new String[realms.size()]));
  }
  
  public void addRealms(File securityConfigurationXml, String... realms) throws IOException {
    Document document = XMLParser.parse(securityConfigurationXml);
    Element e = document.getChild("/security-configuration/realms");
    for (String realm : realms) {
      e.addNode(new Element("realm").setText(realm));
    }
    writeResource(securityConfigurationXml, document);
  }

  //
  // Add a plugin to the Nexus installation
  //
  public void addPlugin(String coord, File workDirectory) throws IOException {
    File pluginsDirectory = new File(workDirectory, "plugin-repository");
    File pluginZip = artifactResolver.resolve(coord);
    archiver.unarchive(pluginZip, pluginsDirectory);
  }

  private void writeResource(File pom, Document document) throws IOException {

    String encoding = document.getEncoding();
    XMLWriter writer = new XMLWriter(encoding != null ? new OutputStreamWriter(new FileOutputStream(pom), encoding) : new OutputStreamWriter(new FileOutputStream(pom)));

    try {
      document.toXML(writer);
    } finally {
      IOUtil.close(writer);
    }
  }

}
