package io.provis.utils;

import io.provis.provision.ProvisioningException;

import java.io.BufferedReader;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.slf4j.Logger;


@Named
@Singleton
public class DefaultIO implements IO {

  @Inject
  private Logger log;

  public void close(Closeable closeable) {
    try {
      closeable.close();
    } catch (IOException e) {
      log.warn("Failed to close {}: {}", closeable, e);
    }
  }

  private void copy(InputStream is, OutputStream os) throws IOException {
    for (byte[] buffer = new byte[16 * 1024];;) {
      int bytes = is.read(buffer);
      if (bytes < 0) {
        break;
      }
      os.write(buffer, 0, bytes);
    }
  }

  public void copy(File src, File dst) {
    try {
      InputStream is = new FileInputStream(src);
      try {
        dst.getAbsoluteFile().getParentFile().mkdirs();

        FileOutputStream fos = new FileOutputStream(dst);
        try {
          copy(is, fos);
        } finally {
          fos.close();
        }
      } finally {
        close(is);
      }
    } catch (IOException e) {
      throw new ProvisioningException("Failed to copy " + src + " to " + dst + ": " + e.getMessage(), e);
    }
  }

  public void copy(URL src, File dst) {
    try {
      InputStream is = src.openStream();
      try {
        dst.getAbsoluteFile().getParentFile().mkdirs();

        FileOutputStream fos = new FileOutputStream(dst);
        try {
          copy(is, fos);
        } finally {
          fos.close();
        }
      } finally {
        close(is);
      }
    } catch (IOException e) {
      throw new ProvisioningException("Failed to copy " + src + " to " + dst + ": " + e.getMessage(), e);
    }
  }

  public String read(File file, String encoding) {
    try {
      InputStream is = new FileInputStream(file);
      try {
        return read(is, encoding);
      } finally {
        close(is);
      }
    } catch (IOException e) {
      throw new ProvisioningException("Failed to read " + file + ": " + e.getMessage(), e);
    }
  }

  public String read(URL url, String encoding) {
    try {
      InputStream is = url.openStream();
      try {
        return read(is, encoding);
      } finally {
        close(is);
      }
    } catch (IOException e) {
      throw new ProvisioningException("Failed to read " + url + ": " + e.getMessage(), e);
    }
  }

  private String read(InputStream is, String encoding) throws IOException {
    Reader reader = null;
    try {
      if (encoding == null || encoding.length() <= 0) {
        reader = new InputStreamReader(is);
      } else {
        reader = new InputStreamReader(is, encoding);
      }
      StringBuilder buffer = new StringBuilder(32 * 1024);
      reader = new BufferedReader(reader);
      for (char[] chars = new char[4 * 1024];;) {
        int read = reader.read(chars);
        if (read < 0) {
          break;
        }
        buffer.append(chars, 0, read);
      }
      return buffer.toString();
    } finally {
      close(reader);
    }
  }

  public void write(File file, String encoding, String text) {
    try {
      file.getAbsoluteFile().getParentFile().mkdirs();
      FileOutputStream fos = new FileOutputStream(file);
      try {
        byte[] bytes;
        if (encoding == null || encoding.length() <= 0) {
          bytes = text.getBytes();
        } else {
          bytes = text.getBytes(encoding);
        }
        fos.write(bytes);
      } finally {
        fos.close();
      }
    } catch (IOException e) {
      throw new ProvisioningException("Failed to write " + file + ": " + e.getMessage(), e);
    }
  }

  public boolean delete(final File path) {
    if (path == null) {
      return true;
    }

    final Collection<File> undeletables = new ArrayList<File>();

    delete(path, undeletables);

    if (!undeletables.isEmpty()) {
      log.warn("Could not delete {}", undeletables);
    }

    return undeletables.isEmpty();
  }

  private void delete(final File file, final Collection<File> undeletables) {
    final String[] children = file.list();
    if (children != null) {
      for (final String child : children) {
        delete(new File(file, child), undeletables);
      }
    }

    if (!file.delete() && file.exists()) {
      undeletables.add(file.getAbsoluteFile());
    }
  }

  public void unzip(File zipFile, File outputDirectory) {
    ZipInputStream is = null;
    try {
      is = new ZipInputStream(new FileInputStream(zipFile));

      for (ZipEntry ze = is.getNextEntry(); ze != null; ze = is.getNextEntry()) {
        File file = new File(outputDirectory, ze.getName());

        if (ze.isDirectory()) {
          file.mkdirs();
        } else {
          file.getParentFile().mkdirs();

          FileOutputStream fos = new FileOutputStream(file);
          try {
            copy(is, fos);
          } finally {
            fos.close();
          }
        }
      }
    } catch (IOException e) {
      throw new ProvisioningException("Failed to unzip " + zipFile + " to " + outputDirectory, e);
    } finally {
      close(is);
    }
  }

  public Properties loadProperties(final File file, final Properties props) {
    Properties result = props;
    if (result == null) {
      result = new Properties();
    }
    try {
      final FileInputStream fis = new FileInputStream(file);
      try {
        result.load(fis);
      } finally {
        close(fis);
      }
    } catch (final IOException e) {
      throw new ProvisioningException("Failed to load properties from " + file + ": " + e.getMessage(), e);
    }
    return result;
  }

  public void saveProperties(final File file, final Properties props) {
    try {
      FileOutputStream fos = new FileOutputStream(file);
      try {
        if (props != null) {
          props.store(fos, "");
        }
      } finally {
        fos.close();
      }
    } catch (final IOException e) {
      throw new ProvisioningException("Failed to save properties to " + file + ": " + e.getMessage(), e);
    }
  }

}
